/**
 * Copyright 2012-2013 University Of Southern California
 * <p>
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * <p>
 * http://www.apache.org/licenses/LICENSE-2.0
 * <p>
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package org.workflowsim;

import org.cloudbus.cloudsim.Cloudlet;
import org.cloudbus.cloudsim.DatacenterBroker;
import org.cloudbus.cloudsim.Log;
import org.cloudbus.cloudsim.core.CloudSim;
import org.cloudbus.cloudsim.core.CloudSimTags;
import org.cloudbus.cloudsim.core.SimEvent;
import org.cloudbus.cloudsim.lists.VmList;
import org.workflowsim.failure.FailureGenerator;
import org.workflowsim.scheduling.*;
import org.workflowsim.utils.Parameters;

import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.JarURLConnection;
import java.net.URL;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

/**
 * WorkflowScheduler represents a algorithm acting on behalf of a user. It hides
 * VM management, as vm creation, sumbission of jobs to this VMs and destruction
 * of VMs. It picks up a scheduling algorithm based on the configuration
 *
 * @author Weiwei Chen
 * @date Apr 9, 2013
 * @since WorkflowSim Toolkit 1.0
 */
public class WorkflowScheduler extends DatacenterBroker {

    /**
     * The workflow engine id associated with this workflow algorithm.
     */
    private int workflowEngineId;

    /**
     * 从包package中获取所有的Class
     *
    // * @param pack
     * @return
     */
//    public static List<Class<?>> getClasses(String packageName) {
//        //第一个class类的集合
//        List<Class<?>> classes = new ArrayList<Class<?>>();
//        //是否循环迭代
//        boolean recursive = true;
//        //获取包的名字 并进行替换
//        String packageDirName = packageName.replace('.', '/');
//        //定义一个枚举的集合 并进行循环来处理这个目录下的things
//        Enumeration<URL> dirs;
//        try {
//            dirs = Thread.currentThread().getContextClassLoader().getResources(packageDirName);
//            //循环迭代下去
//            while (dirs.hasMoreElements()) {
//                //获取下一个元素
//                URL url = dirs.nextElement();
//                //得到协议的名称
//                String protocol = url.getProtocol();
//                //如果是以文件的形式保存在服务器上
//                if ("file".equals(protocol)) {
//                    //获取包的物理路径
//                    String filePath = URLDecoder.decode(url.getFile(), "UTF-8");
//                    //以文件的方式扫描整个包下的文件 并添加到集合中
//                    findAndAddClassesInPackageByFile(packageName, filePath, recursive, classes);
//                } else if ("jar".equals(protocol)) {
//                    //如果是jar包文件
//                    //定义一个JarFile
//                    JarFile jar;
//                    try {
//                        //获取jar
//                        jar = ((JarURLConnection) url.openConnection()).getJarFile();
//                        //从此jar包 得到一个枚举类
//                        Enumeration<JarEntry> entries = jar.entries();
//                        //同样的进行循环迭代
//                        while (entries.hasMoreElements()) {
//                            //获取jar里的一个实体 可以是目录 和一些jar包里的其他文件 如META-INF等文件
//                            JarEntry entry = entries.nextElement();
//                            String name = entry.getName();
//                            //如果是以/开头的
//                            if (name.charAt(0) == '/') {
//                                //获取后面的字符串
//                                name = name.substring(1);
//                            }
//                            //如果前半部分和定义的包名相同
//                            if (name.startsWith(packageDirName)) {
//                                int idx = name.lastIndexOf('/');
//                                //如果以"/"结尾 是一个包
//                                if (idx != -1) {
//                                    //获取包名 把"/"替换成"."
//                                    packageName = name.substring(0, idx).replace('/', '.');
//                                }
//                                //如果可以迭代下去 并且是一个包
//                                if ((idx != -1) || recursive) {
//                                    //如果是一个.class文件 而且不是目录
//                                    if (name.endsWith("Algorithm.class") && !entry.isDirectory()) {
//                                        //去掉后面的".class" 获取真正的类名
//                                        String className = name.substring(packageName.length() + 1, name.length() - 6);
//                                        try {
//                                            //添加到classes
//                                            classes.add(Class.forName(packageName + '.' + className));
//                                        } catch (ClassNotFoundException e) {
//                                            e.printStackTrace();
//                                        }
//                                    }
//                                }
//                            }
//                        }
//                    } catch (IOException e) {
//                        e.printStackTrace();
//                    }
//                }
//            }
//        } catch (IOException e) {
//            e.printStackTrace();
//        }
//        return classes;
//    }

    /**
     * 以文件的形式来获取包下的所有Class
     * @param packageName
     * @param packagePath
     * @param recursive
     * @param classes
     */
//    public static void findAndAddClassesInPackageByFile(String packageName, String packagePath, final boolean recursive, List<Class<?>> classes){
//        //获取此包的目录 建立一个File
//        File dir = new File(packagePath);
//        //如果不存在或者 也不是目录就直接返回
//        if (!dir.exists() || !dir.isDirectory()) {
//            return;
//        }
//        //如果存在 就获取包下的所有文件 包括目录
//        File[] dirfiles = dir.listFiles(new FileFilter() {
//            //自定义过滤规则 如果可以循环(包含子目录) 或则是以.class结尾的文件(编译好的java类文件)
//            public boolean accept(File file) {
//                return (recursive && file.isDirectory()) || (file.getName().endsWith(".class"));
//            }
//        });
//        //循环所有文件
//        for (File file : dirfiles) {
//            //如果是目录 则继续扫描
//            if (file.isDirectory()) {
//                findAndAddClassesInPackageByFile(packageName + "." + file.getName(),
//                        file.getAbsolutePath(),
//                        recursive,
//                        classes);
//            }
//            else {
//                String name = file.getName();
//                if (name.endsWith("Algorithm.class")) {
//                    //如果是java类的算法文件 去掉后面的.class 只留下类名
//                    String className = file.getName().substring(0, file.getName().length() - 6);
//                    try {
//                        //添加到集合中去
//                        classes.add(Class.forName(packageName + '.' + className));
//                    } catch (ClassNotFoundException e) {
//                        e.printStackTrace();
//                    }
//                }
//            }
//        }
//    }


    /**
     * Created a new WorkflowScheduler object.
     *
     * @param name name to be associated with this entity (as required by
     *             Sim_entity class from simjava package)
     * @throws Exception the exception
     * @pre name != null
     * @post $none
     */
    public WorkflowScheduler(String name) throws Exception {
        super(name);
//        for (int i = 0; i < AlgorithmList.size(); ++i) {
//            Class<?> cl = AlgorithmList.get(i);
//
//        }
    }

    /**
     * Binds this scheduler to a datacenter
     *
     * @param datacenterId data center id
     */
    public void bindSchedulerDatacenter(int datacenterId) {
        if (datacenterId <= 0) {
            Log.printLine("Error in data center id");
            return;
        }
        this.datacenterIdsList.add(datacenterId);
    }

    /**
     * Sets the workflow engine id
     *
     * @param workflowEngineId the workflow engine id
     */
    public void setWorkflowEngineId(int workflowEngineId) {
        this.workflowEngineId = workflowEngineId;
    }

    /**
     * Process an event
     *
     * @param ev a simEvent obj
     */
    @Override
    public void processEvent(SimEvent ev) throws ClassNotFoundException {
        switch (ev.getTag()) {
            // Resource characteristics request
            case CloudSimTags.RESOURCE_CHARACTERISTICS_REQUEST:
                processResourceCharacteristicsRequest(ev);
                break;
            // Resource characteristics answer
            case CloudSimTags.RESOURCE_CHARACTERISTICS:
                processResourceCharacteristics(ev);
                break;
            // VM Creation answer
            case CloudSimTags.VM_CREATE_ACK:
                processVmCreate(ev);
                break;
            // A finished cloudlet returned
            case WorkflowSimTags.CLOUDLET_CHECK:
                processCloudletReturn(ev);
                break;
            case CloudSimTags.CLOUDLET_RETURN:
                processCloudletReturn(ev);
                break;
            case CloudSimTags.END_OF_SIMULATION:
                shutdownEntity();
                break;
            case CloudSimTags.CLOUDLET_SUBMIT:
                processCloudletSubmit(ev);
                break;
            case WorkflowSimTags.CLOUDLET_UPDATE:
                processCloudletUpdate(ev);
                break;
            default:
                processOtherEvent(ev);
                break;
        }
    }

//    /**
//     * Switch between multiple schedulers. Based on algorithm.method
//     *
//     * @param name the SchedulingAlgorithm name
//     * @return the algorithm that extends BaseSchedulingAlgorithm
//     */
//    private BaseSchedulingAlgorithm getScheduler(SchedulingAlgorithm name) {
//        BaseSchedulingAlgorithm algorithm;
//
//        // choose which algorithm to use. Make sure you have add related enum in
//        //Parameters.java
//        switch (name) {
//            //by default it is Static
//            case FCFS:
//                algorithm = new FCFSSchedulingAlgorithm();
//                break;
//            case MINMIN:
//                algorithm = new MinMinSchedulingAlgorithm();
//                break;
//            case MAXMIN:
//                algorithm = new MaxMinSchedulingAlgorithm();
//                break;
//            case MCT:
//                algorithm = new MCTSchedulingAlgorithm();
//                break;
//            case DATA:
//                algorithm = new DataAwareSchedulingAlgorithm();
//                break;
//            case STATIC:
//                algorithm = new StaticSchedulingAlgorithm();
//                break;
//            case ROUNDROBIN:
//                algorithm = new RoundRobinSchedulingAlgorithm();
//                break;
//            default:
//                algorithm = new StaticSchedulingAlgorithm();
//                break;
//
//        }
//        return algorithm;
//    }

    /**
     * Switch between multiple schedulers. Based on algorithm.method
     *
     //* @param name the SchedulingAlgorithm name
     * @return the algorithm that extends BaseSchedulingAlgorithm
     */
    private BaseSchedulingAlgorithm getScheduler(String AlgorithmName) throws ClassNotFoundException {
        BaseSchedulingAlgorithm algorithm = null;
        Class cls=Class.forName("org.workflowsim.scheduling"+"."+AlgorithmName);
        try {
            algorithm = (BaseSchedulingAlgorithm) cls.newInstance();
        }catch (IllegalAccessException | InstantiationException e){
            System.out.println("algorithm = (BaseSchedulingAlgorithm) cls.newInstance();");
        }
        return algorithm;
    }

    /**
     * Process the ack received due to a request for VM creation.
     *
     * @param ev a SimEvent object
     * @pre ev != null
     * @post $none
     */
    @Override
    protected void processVmCreate(SimEvent ev) {
        int[] data = (int[]) ev.getData();
        int datacenterId = data[0];
        int vmId = data[1];
        int result = data[2];

        if (result == CloudSimTags.TRUE) {
            getVmsToDatacentersMap().put(vmId, datacenterId);
            /**
             * Fix a bug of cloudsim Don't add a null to getVmsCreatedList()
             * June 15, 2013
             */
            if (VmList.getById(getVmList(), vmId) != null) {
                getVmsCreatedList().add(VmList.getById(getVmList(), vmId));
                Log.printLine(CloudSim.clock() + ": " + getName() + ": VM #" + vmId
                        + " has been created in Datacenter #" + datacenterId + ", Host #"
                        + VmList.getById(getVmsCreatedList(), vmId).getHost().getId());
            }
        } else {
            Log.printLine(CloudSim.clock() + ": " + getName() + ": Creation of VM #" + vmId
                    + " failed in Datacenter #" + datacenterId);
        }

        incrementVmsAcks();

        // all the requested VMs have been created
        if (getVmsCreatedList().size() == getVmList().size() - getVmsDestroyed()) {
            submitCloudlets();
        } else {
            // all the acks received, but some VMs were not created
            if (getVmsRequested() == getVmsAcks()) {
                // find id of the next datacenter that has not been tried
                for (int nextDatacenterId : getDatacenterIdsList()) {
                    if (!getDatacenterRequestedIdsList().contains(nextDatacenterId)) {
                        createVmsInDatacenter(nextDatacenterId);
                        return;
                    }
                }

                // all datacenters already queried
                if (getVmsCreatedList().size() > 0) { // if some vm were created
                    submitCloudlets();
                } else { // no vms created. abort
                    Log.printLine(CloudSim.clock() + ": " + getName()
                            + ": none of the required VMs could be created. Aborting");
                    finishExecution();
                }
            }
        }
    }

    /**
     * Update a cloudlet (job)
     *
     * @param ev a simEvent object
     */
    protected void processCloudletUpdate(SimEvent ev) throws ClassNotFoundException {
        BaseSchedulingAlgorithm scheduler = getScheduler(Parameters.AlgorithmName);
        scheduler.setCloudletList(getCloudletList());
        scheduler.setVmList(getVmsCreatedList());

        try {
            scheduler.run();
        } catch (Exception e) {
            Log.printLine("Error in configuring scheduler_method");
            e.printStackTrace();
        }

        List<Cloudlet> scheduledList = scheduler.getScheduledList();
        for (Cloudlet cloudlet : scheduledList) {
            int vmId = cloudlet.getVmId();
            double delay = 0.0;
            if (Parameters.getOverheadParams().getQueueDelay() != null) {
                delay = Parameters.getOverheadParams().getQueueDelay(cloudlet);
            }
            schedule(getVmsToDatacentersMap().get(vmId), delay, CloudSimTags.CLOUDLET_SUBMIT, cloudlet);
        }
        getCloudletList().removeAll(scheduledList);
        getCloudletSubmittedList().addAll(scheduledList);
        cloudletsSubmitted += scheduledList.size();
    }
    /**
     * Process a cloudlet (job) return event.
     *
     * @param ev a SimEvent object
     * @pre ev != $null
     * @post $none
     */
    @Override
    protected void processCloudletReturn(SimEvent ev) {
        Cloudlet cloudlet = (Cloudlet) ev.getData();
        Job job = (Job) cloudlet;

        /**
         * Generate a failure if failure rate is not zeros.
         */
        FailureGenerator.generate(job);

        getCloudletReceivedList().add(cloudlet);
        getCloudletSubmittedList().remove(cloudlet);

        CondorVM vm = (CondorVM) getVmsCreatedList().get(cloudlet.getVmId());
        //so that this resource is released
        vm.setState(WorkflowSimTags.VM_STATUS_IDLE);

        double delay = 0.0;
        if (Parameters.getOverheadParams().getPostDelay() != null) {
            delay = Parameters.getOverheadParams().getPostDelay(job);
        }
        schedule(this.workflowEngineId, delay, CloudSimTags.CLOUDLET_RETURN, cloudlet);

        cloudletsSubmitted--;
        //not really update right now, should wait 1 s until many jobs have returned
        schedule(this.getId(), 0.0, WorkflowSimTags.CLOUDLET_UPDATE);

    }

    /**
     * Start this entity (WorkflowScheduler)
     */
    @Override
    public void startEntity() {
        Log.printLine(getName() + " is starting...");
        // this resource should register to regional GIS.
        // However, if not specified, then register to system GIS (the
        // default CloudInformationService) entity.
        //int gisID = CloudSim.getEntityId(regionalCisName);
        int gisID = -1;
        if (gisID == -1) {
            gisID = CloudSim.getCloudInfoServiceEntityId();
        }

        // send the registration to GIS
        sendNow(gisID, CloudSimTags.REGISTER_RESOURCE, getId());
    }

    /**
     * Terminate this entity (WorkflowScheduler)
     */
    @Override
    public void shutdownEntity() {
        clearDatacenters();
        Log.printLine(getName() + " is shutting down...");
    }

    /**
     * Submit cloudlets (jobs) to the created VMs. Scheduling is here
     */
    @Override
    protected void submitCloudlets() {
        sendNow(this.workflowEngineId, CloudSimTags.CLOUDLET_SUBMIT, null);
    }

    /**
     * A trick here. Assure that we just submit it once
     */
    private boolean processCloudletSubmitHasShown = false;

    /**
     * Submits cloudlet (job) list
     *
     * @param ev a simEvent object
     */
    protected void processCloudletSubmit(SimEvent ev) {
        List<Job> list = (List) ev.getData();
        getCloudletList().addAll(list);

        sendNow(this.getId(), WorkflowSimTags.CLOUDLET_UPDATE);
        if (!processCloudletSubmitHasShown) {
            processCloudletSubmitHasShown = true;
        }
    }

    /**
     * Process a request for the characteristics of a PowerDatacenter.
     *
     * @param ev a SimEvent object
     * @pre ev != $null
     * @post $none
     */
    @Override
    protected void processResourceCharacteristicsRequest(SimEvent ev) {
        setDatacenterCharacteristicsList(new HashMap<>());
        Log.printLine(CloudSim.clock() + ": " + getName() + ": Cloud Resource List received with "
                + getDatacenterIdsList().size() + " resource(s)");
        for (Integer datacenterId : getDatacenterIdsList()) {
            sendNow(datacenterId, CloudSimTags.RESOURCE_CHARACTERISTICS, getId());
        }
    }
}
